/*
    NOTE: This example is the same as "lots_of_threads", but with tracy set up for performance profiling.

    How to run:

    1. Install tracy
       - Follow the instructions at https://github.com/wolfpld/tracy
       - Or install via something like homebrew: `brew install tracy`
    2. Run the server (`tracy`) in a terminal
    3. Run the example in another terminal
       - `export RUST_LOG=trace`
       - `cargo run --example tracy`
*/

#![allow(clippy::missing_errors_doc)]

use std::time::Duration;

use async_io::{block_on, Timer};

use mlua::prelude::*;
use mlua_luau_runtime::{Functions, Runtime};

const MAIN_SCRIPT: &str = include_str!("./lua/lots_of_threads.luau");

const ONE_NANOSECOND: Duration = Duration::from_nanos(1);

pub fn main() -> LuaResult<()> {
    use tracing_subscriber::layer::SubscriberExt;
    tracing::subscriber::set_global_default(
        tracing_subscriber::registry().with(tracing_tracy::TracyLayer::default()),
    )
    .unwrap();

    let _client = tracing_tracy::client::Client::start();

    // Set up persistent Lua environment
    let lua = Lua::new();
    let rt = Runtime::new(&lua);
    let fns = Functions::new(&lua)?;

    lua.globals().set("spawn", fns.spawn)?;
    lua.globals().set(
        "sleep",
        lua.create_async_function(|_, ()| async move {
            // Obviously we can't sleep for a single nanosecond since
            // this uses OS scheduling under the hood, but we can try
            Timer::after(ONE_NANOSECOND).await;
            Ok(())
        })?,
    )?;

    // Load the main script into the runtime
    let main = lua.load(MAIN_SCRIPT);
    rt.push_thread_front(main, ())?;

    // Run until completion
    block_on(rt.run());

    Ok(())
}
